#include "stdtypes.h"

/* Inverse DCT Coefficient Table */
static long IDctCoeff[8][8] = {  
	{4096, 5681, 5352, 4816, 4096, 3218, 2217, 1130},
	{4096, 4816, 2217,-1130,-4096,-5681,-5352,-3218},
	{4096, 3218,-2217,-5681,-4096, 1130, 5352, 4816},
	{4096, 1130,-5352,-3218, 4096, 4816,-2217,-5681},
	{4096,-1130,-5352, 3218, 4096,-4816,-2217, 5681},
	{4096,-3218,-2217, 5681,-4096,-1130, 5352,-4816},
	{4096,-4816, 2217, 1130,-4096, 5681,-5352, 3218},
	{4096,-5681, 5352,-4816, 4096,-3218, 2217,-1130}
};

/* Forward DCT Coefficient Table */
static long DctCoeff[8][8] = { 
	{4096, 4096, 4096, 4096, 4096, 4096, 4096, 4096},
	{5681, 4816, 3218, 1130,-1130,-3218,-4816,-5681},
	{5352, 2217,-2217,-5352,-5352,-2217, 2217, 5352},
	{4816,-1130,-5681,-3218, 3218, 5681, 1130,-4816},
	{4096,-4096,-4096, 4096, 4096,-4096,-4096, 4096},
	{3218,-5681, 1130, 4816,-4816,-1130, 5681,-3218},
	{2217,-5352, 5352,-2217,-2217, 5352,-5352, 2217},
	{1130,-3218, 4816,-5681, 5681,-4816, 3218,-1130}
};

/****************************************************************************
 Objectives : (block/2**round) and take only no. of "select" bits
 Input      : a block, round, select
 Output     : DCT block
 Called By  : dct(), idct()
 Calls      : none
****************************************************************************/
long *selector(block, round, select)
long block[][8];
int round, select;
{

	static unsigned long maskTable[] =
	{
		0x00000000L, 0x00000001L, 0x00000003L, 0x00000007L,
		0x0000000FL, 0x0000001FL, 0x0000003FL, 0x0000007FL,
		0x000000FFL, 0x000001FFL, 0x000003FFL, 0x000007FFL,
		0x00000FFFL, 0x00001FFFL, 0x00003FFFL, 0x00007FFFL,
		0x0000FFFFL, 0x0001FFFFL, 0x0003FFFFL, 0x0007FFFFL,
		0x000FFFFFL, 0x001FFFFFL, 0x003FFFFFL, 0x007FFFFFL,
		0x00FFFFFFL, 0x01FFFFFFL, 0x03FFFFFFL, 0x07FFFFFFL,
		0x0FFFFFFFL, 0x1FFFFFFFL, 0x3FFFFFFFL, 0x7FFFFFFFL
	};
	long 	data ,overflow;
	int 	i, j;

	select--;
	for (i=0; i<8; i++)
	   for (j=0; j<8; j++) {
		data = block[i][j];		
		/* check data whether it is a negative or positive one */
		if ((data & maskTable[round] & ~maskTable[round-1]) != 0)
			data = (data>>round)+1;  
		else  /* positive */
			data = data>>round;
		overflow = data & ~maskTable[select];
		if (data>=0)
		{
			if (overflow != 0)
				/* saturate to maxpoint */
				data = maskTable[select];  
		}
		else
		{
			if (overflow != ~maskTable[select])
				/* saturate to minpoint */
				data = ~maskTable[select];
		}
		block[i][j] = data;
	   }
	return (long *)block;
};

/****************************************************************************
 Objectives : Forward DCT of a block
 Input      : a block to be DCT
 Output     : DCT block
 Called By  : mcu_coding()
 Calls      : matrix_multiplication(), selector(), transpose()
****************************************************************************/
long *dct(in, transformed) 
long  in[][8], transformed[][8];
{
	long	block[8][8], tmp[8][8];	/* temporary blocks */
	int     i, j;			/* index for loops  */

	matrix_multiplication(in, DctCoeff, block);
	selector(block, 9, 15); /* select 15 bits */
	transpose(block);
	matrix_multiplication(block, DctCoeff, tmp);
	selector(tmp, 18, 12);  /* select 12 bits */
	transpose(tmp);
	for(i=0; i<8; i++)
	   for(j=0; j<8; j++)
	      transformed[i][j] = tmp[i][j]; 
	return (long *)transformed;
};

/****************************************************************************
 Objectives : Inverse DCT of a block
 Input      : a block to be IDCT
 Output     : IDCT block
 Called By  : mcu_coding()
 Calls      : matrix_multiplication(), selector(), transpose()
****************************************************************************/
long *idct(in) 
long  in[][8];
{
	long	block[8][8], tmp[8][8];	/* temporary blocks */
	int	i, j;			/* index for loops  */

	matrix_multiplication(in, IDctCoeff, block);
	selector(block, 9, 15);  /* select 15 bits */
	transpose(block);
	matrix_multiplication(block, IDctCoeff, tmp);
	selector(tmp, 18, 12);   /* select 12 bits */
	transpose(tmp);
	for(i=0; i<8; i++)
	   for(j=0; j<8; j++)
	      in[i][j] = tmp[i][j]; 
	return (long *)in;
};
